package com.alibaba.dataworks.udtf;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import com.aliyun.odps.udf.ExecutionContext;
import com.aliyun.odps.udf.UDFException;
import com.aliyun.odps.udf.UDTF;
import com.aliyun.odps.udf.annotation.Resolve;
import com.aliyun.odps.utils.StringUtils;
// TODO define input and output types, e.g. "string,string->string,bigint".
@Resolve({"string, string, string, string, string, string -> string, string, string, string, string, string, string, string, string, string, string,string"})
public class AdviserScoreUDTF extends UDTF {

    @Override
    public void setup(ExecutionContext ctx) throws UDFException {

    }

    @Override
    public void process(Object[] args) throws UDFException {
        Adviser adviser = new Adviser(args);
        //发送im消息总数
        Integer Indicators1 = 0;
        //视频看房平均通话时长
        Double Indicators2 = 0.0;
        //视频看房平均响应时长
        Double Indicators3 = 0.0;
        //电话平均通话时长
        Double Indicators4 = 0.0;
        // 点击抢单总数
        Integer Indicators5 = 0;
        // 平均把控分
        Double Indicators6 = 0.0;
        String score = "";
        int index = 0;
        for(String idValue: adviser.getIdValuesList()) {
            if("response_time".equals(idValue)) {
                // 顾问的平均响应时长
                String[] dataValuesList = adviser.getValueDatasList();
                if (dataValuesList.length > 0 && index < dataValuesList.length) {
                    Indicators3 = Double.parseDouble(dataValuesList[index]);
                }

            } else if ("clk_2b_25".equals(idValue) || "clk_2cmina_27".equals(idValue)) {
                // 发送im消息总数
                String [] dataValuesList = adviser.getCountDatasList();
                if (dataValuesList.length > 0 && index < dataValuesList.length) {
                    String[] imCount = dataValuesList[index].split(",");
                    for (String count: imCount) {
                        Indicators1 += Integer.parseInt(count);
                    }
                }
            } else if ("im_call".equals(idValue)) {
                // 视频看房平均通话时长
                String[] videoTimeList = adviser.getValueDatasList();
                if(videoTimeList.length > 0 && index < videoTimeList.length) {
                    String[] videoList = videoTimeList[index].split(",");
                    for(String time: videoList) {
                        Indicators2 += diffTime(time);
                    }

                    if(Indicators2 > 0) {
                        Indicators2 = Indicators2 / videoList.length;
                    }
                }

            } else if ("adviser_score".equals(idValue)) {
                // 平均把控分 Indicators6
                String[] adviserScores = adviser.getValueDatasList();
                if(adviserScores.length > 0 && index < adviserScores.length) {
                    String[] adviserscoreList = adviserScores[index].split(",");
                    for(String adviserScore: adviserscoreList) {
                        Indicators6 += Double.parseDouble(adviserScore);
                    }

                    if (Indicators6 > 0 && adviserscoreList.length > 0) {
                        Indicators6 = Indicators6 / adviserscoreList.length;
                    }
                }
            }
            index++;
        }
        score = score(Indicators1, Indicators2, Indicators3, Indicators4, Indicators5, Indicators6);
        forward(adviser.getBrandId(), adviser.getHouseId(), adviser.getUserId(), Indicators1 + "", Indicators2 + "", Indicators3 + "", Indicators4 + "", Indicators5 + "", Indicators6 + "", score, "", date());

    }

    private String score(Integer one, Double two, Double three, Double four, Integer five, Double six) {
        Integer score = imInfoScore(one) + videoSeeHouseCallTime(two) + videoSeeHouseResponseTime(three) + phoneWorkTimeAVG(four) + grabASingle(five) + avgControlScore(six);

        return score*100/120 + "";
    }


    private String date() {
        Date date = new Date();
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        return format.format(date);
    }


    @Override
    public void close() throws UDFException {

    }

    private Integer avgControlScore(Double six) {
        if (six >= 80 && six <= 100) {
            return 20;
        } else if (six >= 60 && six < 79) {
            return 10;
        } else if (six > 0 && six < 59) {
            return 5;
        }
        return 0;
    }

    private Integer grabASingle(Integer five) {
        return 0;
    }

    private Integer phoneWorkTimeAVG(Double four) {

        return 0;
    }

    private Integer videoSeeHouseResponseTime(Double three) {
        if (three > 0 && three < 30) {
            return 20;
        } else if (three >= 30 && three <= 60) {
            return 10;
        } else if (three > 60) {
            return 5;
        }
        return 0;
    }

    private Integer videoSeeHouseCallTime(Double two) {
        if (two > 5*60) {
            return 20;
        } else if (two <= 5*60 && two > 3*60) {
            return 10;
        } else if (two > 1*60 && two <= 3*60) {
            return 5;
        }

        return 0;
    }

    private Integer imInfoScore(Integer one) {
        if (one > 8) {
            return 20;
        } else if (one > 5 && one <= 8) {
            return 10;
        } else if (one > 0 && one <= 5) {
            return 5;
        }
        return 0;
    }


    private long diffTime(String endTime) {
        try {
            DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Date startDate = df.parse("1970-01-01 00:00:00");
            Date endDate = df.parse(endTime);
            long diff = endDate.getTime() - startDate.getTime();
            if (diff > 0) {
                return diff / 1000;
            } else {
                return 0L;
            }
        } catch(Exception e) {
            return 0L;
        }
    }

    private String getValue(Object[] args, int index) {
        return (String)args[index];
    }

    public static class Adviser {
        String brandId;
        String houseId;
        String userId;
        String idValues;
        String countDatas;
        String valueDatas;
        String[] idValuesList;
        String[] countDatasList;
        String[] valueDatasList;


        public Adviser(Object[] args) {
            this.brandId = getValue(args, 0);
            this.houseId =  getValue(args, 1);
            this.userId = getValue(args, 2);
            this.idValues = getValue(args, 3);
            this.countDatas = getValue(args, 4);
            this.valueDatas = getValue(args, 5);
            this.idValuesList = this.idValues.split("~");
            this.countDatasList = this.countDatas.split("~");
            if(!StringUtils.isNullOrEmpty(this.valueDatas)) {
                this.valueDatasList = this.valueDatas.split("~");
            } else {
                this.valueDatasList = new String[0];
            }
        }

        public String getBrandId() {
            return brandId;
        }

        public void setBrandId(String brandId) {
            this.brandId = brandId;
        }

        public String getHouseId() {
            return houseId;
        }

        public void setHouseId(String houseId) {
            this.houseId = houseId;
        }

        public String getUserId() {
            return userId;
        }

        public void setUserId(String userId) {
            this.userId = userId;
        }

        public String getIdValues() {
            return idValues;
        }

        public void setIdValues(String idValues) {
            this.idValues = idValues;
        }

        public String getCountDatas() {
            return countDatas;
        }

        public void setCountDatas(String countDatas) {
            this.countDatas = countDatas;
        }

        public String getValueDatas() {
            return valueDatas;
        }

        public void setValueDatas(String valueDatas) {
            this.valueDatas = valueDatas;
        }

        public String[] getIdValuesList() {
            return idValuesList;
        }

        public void setIdValuesList(String[] idValuesList) {
            this.idValuesList = idValuesList;
        }

        public String[] getCountDatasList() {
            return countDatasList;
        }

        public void setCountDatasList(String[] countDatasList) {
            this.countDatasList = countDatasList;
        }

        public String[] getValueDatasList() {
            return valueDatasList;
        }

        public void setValueDatasList(String[] valueDatasList) {
            this.valueDatasList = valueDatasList;
        }

        private String getValue(Object[] args, int index) {
            return (String)args[index];
        }
    }

}
